(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright (c) 2013-2014 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: refinement_types.mli,v 1.31 2014/06/10 15:08:34 fclement Exp $ *)

type component_number = Unatural.t;;

(** {6 External representation of parameters}
 Extensional types for parameter refinement. *)

type point = Unatural.t * Unatural.t;;
(** A [point] denotes a matrix entry. *)

type 'a parameter = 'a Umatrix_structural_mapping.mapping
and 'a sub_parameter = 'a Umatrix_structural_mapping.sub_mapping
;;
(** An ['a parameter] is the graph of the function that associates each
 point of the domain of a parameter with ['a] values. *)

type 'a gradient = 'a parameter;;
(** The gradient of a scalar function with respect to a parameter has the same
 representation as the parameter itself. *)

(** {6 Internal representation of parameters}
 Structural types for parameter refinement. *)

type 'a indicator = 'a
and 'a first_order_indicator = 'a indicator
and 'a exact_indicator = 'a indicator
;;
(** The type for first order and exact indicators. *)

type 'a candidate_cutting =
  {
   cutting : Umatrix_nonempty_set.dichotomy;
   first_order_indicator : 'a first_order_indicator;
  }
;;
(** An ['a candidate_cutting] is the association of a dichotomy of the support
 of a sub-part of a parameter with the corresponding first order indicator. *)

type 'a zone =
  ('a, 'a candidate_cutting list) Umatrix_structural_mapping.block
;;
(** An ['a zone] is the association of a sub-part of a parameter with a list of
 candidate cuttings. The evaluation of the list may be delayed. *)

type zone_number = Umatrix_structural_mapping.block_number;;
(** Zones are numbered. *)

type bounds = { ll : point; ur : point; };;
(** Bounds of zones are represented by lower left and upper right corners
 of the smallest rectangle containing the support of the zone. *)

type 'a zonation =
  ('a, 'a candidate_cutting list, 'a gradient) Umatrix_structural_mapping.t
;;
(** An ['a zonation] is a collection of zones.
 The intention is to only consider zones whith disjoint supports.
 ['a] will always be [scalar].
 The evaluation of the lists of candidate cuttings may be delayed. *)

type 'a forced_zone =
  ('a sub_parameter, 'a candidate_cutting list) Udynamic_graph.arrow
;;

type 'a forced_zonation = 'a forced_zone array;;

type 'a partition = 'a sub_parameter array;;

(** {6 Refinement algorithm} *)

type command_line =  string;;
type external_program_command = command_line;;

type dimension = Unatural.t;;

type dimension_management =
  | Vector              (* = (Best_cutting, All_components) *)
  | Best_component_only (* = (Best_cutting, Best_component) *)
;;

type number_of_cuts = Unatural.t;;

type basic_cutting_family =
  | Horizontal_cut
  | Vertical_cut
;;

type cutting_strategy =
  | Best_in_a_family of number_of_cuts * basic_cutting_family Ulist1.list1
  | Overall_best
;;

type candidate_cuttings_selection_criterion = float * Unatural.t;;

type minimum_gradient = float;;

type stopping_criterion =
  {
   max_iteration : Uhistory_types.state_number;
   min_best_scalar_indicator : float;
   min_scalar_relative_error : float;
  }
;;

type graphics_mode =
  | No_graphics        (* = None *)
  | Parameter_graphics (* = Some (Param_quantity, All_channels) *)
  | Full_graphics      (* = Some (All_quantities, All_channels) *)
;;

type output_status =
  | Max_iteration of Uhistory_types.state_number
  | Min_scalar_indicator of float
  | Min_scalar_relative_error of float
  | No_more_cuttings
;;
